package MuGeN::Gui::FeatureWindow;

use strict;

use Bio::SeqFeatureI;
use MuGeN::PhysicalMap;

sub new {
    my $class=shift;
    my $manager=shift;
    my $self={};

    $self->{manager}=$manager;

    my $window=new Gtk2::Window -toplevel;
    $window->set_size_request(450,400);
    $window->set_title("Highlights / Feature Information");
    $window->signal_connect('delete_event',sub { $self->{manager}->change_window_visibility('featurewindow',1); return 1});
    $window->signal_connect('destroy_event',sub { $self->{manager}->change_window_visibility('featurewindow',1); return 1});
    $self->{window}=$window;


    my $paned=new Gtk2::VPaned();
    $paned->show();
    $window->add($paned);

    my $frame=new Gtk2::Frame 'Map Highlights';
    $frame->show();
    $paned->add1($frame);

    

    my $hlightscrolledwindow=new Gtk2::ScrolledWindow;
    $hlightscrolledwindow->show();
    $frame->add($hlightscrolledwindow);

    
    my $model=Gtk2::TreeStore->new("Glib::String");
    my $highlights=Gtk2::TreeView->new_with_model($model);
    $highlights->insert_column_with_attributes(0,'HighLights',Gtk2::CellRendererText->new(),'text',0);
    $highlights->set_headers_visible(0);
    $highlights->show();
    $highlights->get_selection()->signal_connect('changed', sub { $self->highlight_selected(@_) });
    $self->{highlights}=$highlights;
    $hlightscrolledwindow->add_with_viewport($highlights);

    $frame=new Gtk2::Frame 'Feature Qualifiers';
    $frame->show();
    $paned->add2($frame);
    
    my $qualscrolledwindow=new Gtk2::ScrolledWindow;
    $qualscrolledwindow->show();
    $qualscrolledwindow->set_policy('never','automatic');
    $frame->add($qualscrolledwindow);

    my $annotationtext=Gtk2::TextView->new_with_buffer(Gtk2::TextBuffer->new());
    $annotationtext->show();
    $annotationtext->set_wrap_mode('char');
    $qualscrolledwindow->add($annotationtext);
    $self->{annotationtext}=$annotationtext;

    bless $self,$class;

    return $self;
}

sub reactive_zone_entered {
    my $self=shift;
    my ($x,$y)=(shift,shift);
    my $userinfo=shift;
    my $textbuffer='';
    
    if (UNIVERSAL::isa($userinfo,'UNIVERSAL') &&
		       $userinfo->isa('Bio::SeqFeatureI')) {
	$textbuffer="Feature : ".$userinfo->primary_tag()."\n";
	$textbuffer.="Location : ".$userinfo->location->to_FTstring()."\n";
	foreach my $tag ($userinfo->all_tags()) {
	    $textbuffer.="\t/$tag = ".join(' ',$userinfo->each_tag_value($tag))."\n";
	}
    } elsif (defined $userinfo->{info}) {
	$textbuffer=$userinfo->{info};
    }
    
    my $buffer=$self->{annotationtext}->get_buffer();
    $buffer->set_text($textbuffer);
}


sub show_all {
    my $self=shift;

    $self->{window}->show_all;
}

sub build_highlights {
    my $self=shift;


    my $highlights=$self->{highlights};
    my $model=$highlights->get_model();
    $model->clear();
    my $mapinfos=$self->{manager}->get_mapinfo();
    $self->{highlighinfos}={};
    foreach my $mapinfo (@{$mapinfos}) {
	my $mapid=$mapinfo->get_map_id();
	my $iter=$model->append(undef);
	$model->set($iter,0,$mapinfo->get_name());
	my $map=$mapinfo->get_map();
	foreach my $companalid ($map->get_companal_result_display_ids()) {	
	    my $subiter=$model->append($iter);
	    $model->set($subiter,0,$companalid);
	    foreach my $highlight ($map->get_companal_highlight_names($companalid)) {
		my $subsubiter=$model->append($subiter);
		$model->set($subsubiter,0,$highlight);
		my $string=$model->get_string_from_iter($subsubiter);
		$self->{highlightinfos}->{$string}={-mapid => $mapid,
						    -companalid => $companalid,
						    -highlight => $highlight};

	    }
	}
    }
    $highlights->show_all();
}

sub highlight_selected {
    my $self=shift;
    my $selection=shift;
    
    if ($selection->count_selected_rows()) {
	my ($model,$iter)=$selection->get_selected();
	
	my $string=$model->get_string_from_iter($iter);
	
	if (defined $self->{highlightinfos}->{$string}) {
	    my $mapid=$self->{highlightinfos}->{$string}->{-mapid};
	    my $companalid=$self->{highlightinfos}->{$string}->{-companalid};
	    my $highlight=$self->{highlightinfos}->{$string}->{-highlight};
	    
	    $self->{manager}->focus_on_highlight($mapid,$companalid,$highlight);
	}
    }

    return 1;

}

sub destroy {
    my $self=shift;

    $self->{window}->destroy();

}

1
