package MuGeN::PhysicalMapStripSet;

use strict;

use MuGeN::PhysicalMapStrip;

$MuGeN::PhysicalMapStripSet::AXISINDEX=0;
$MuGeN::PhysicalMapStripSet::FEATUREINDEX=1;

$MuGeN::PhysicalMapStripSet::BIRDSEYE_VIEW=1;
$MuGeN::PhysicalMapStripSet::FEATURE_VIEW=2;
$MuGeN::PhysicalMapStripSet::SEQUENCE_VIEW=3;

sub _determine_view_mode {
    my $basespan=shift;

    $MuGeN::PhysicalMapStripSet::view_mode=$MuGeN::PhysicalMapStripSet::BIRDSEYE_VIEW
	if ($basespan>=$MuGeN::PhysicalMapStripSet::BIRD_FEATURE_THRESH);

    $MuGeN::PhysicalMapStripSet::view_mode=$MuGeN::PhysicalMapStripSet::FEATURE_VIEW
	if ($basespan<$MuGeN::PhysicalMapStripSet::BIRD_FEATURE_THRESH &&
	    $basespan>$MuGeN::PhysicalMapStripSet::FEATURE_SEQUENCE_THRESH);

    $MuGeN::PhysicalMapStripSet::view_mode=$MuGeN::PhysicalMapStripSet::SEQUENCE_VIEW
	if ($basespan<=$MuGeN::PhysicalMapStripSet::FEATURE_SEQUENCE_THRESH);
	    
}

sub new {
    my $class = shift;
    my $self = {};
    my $map=shift;
    my $startbase=shift;
    my $endbase=shift;

    $self->{startbase}=$startbase;
    $self->{endbase}=$endbase;
    $self->{strips}=[];

    $self->{strips}->[$MuGeN::PhysicalMapStripSet::AXISINDEX]=new MuGeN::AxisStrip($map,$startbase,$endbase);
    $self->{strips}->[$MuGeN::PhysicalMapStripSet::FEATUREINDEX]=new MuGeN::FeatureStrip($map,$startbase,$endbase);

    $self->{axiscompanalresultwidgets} = [];
    $self->{scalablecompanalresultwidgets} = [];
    $self->{interlacecompanalresultwidgets} = [];
    $self->{backgroundcolor}='yellow';

    bless $self,$class;
    return $self;
    
}

sub add_feature_widget {
    my $self=shift;
    my $widget=shift;
    my $axis_strip=$self->{strips}->[$MuGeN::PhysicalMapStripSet::AXISINDEX];
    my $scalable_strip=$self->{strips}->[$MuGeN::PhysicalMapStripSet::FEATUREINDEX];
    if (defined $widget && $widget->isa('MuGeN::PositionalFeatureWidget')) {
	$axis_strip->add_feature_widget($widget);
    }
    if (defined $widget && $widget->isa('MuGeN::ScalableFeatureWidget')) {
	$scalable_strip->add_feature_widget($widget);
    }
}

sub add_companal_result_widget {
    my $self=shift;
    my $widget=shift;

    if ($widget->layout eq 'axis_overlay') {
	push @{$self->{axiscompanalresultwidgets}},{widget=>$widget,
							strip=>$self->{strips}->[$MuGeN::PhysicalMapStripSet::AXISINDEX]};
    }

    if ($widget->layout eq 'feature_overlay' || $widget->layout eq 'overlay'){
	push @{$self->{scalablecompanalresultwidgets}},{widget=>$widget,
							strip=>$self->{strips}->[$MuGeN::PhysicalMapStripSet::FEATUREINDEX]};
    }
    if ($widget->layout eq 'sequence_overlay') {
	push @{$self->{axiscompanalresultwidgets}},{widget=>$widget,
							strip=>$self->{strips}->[$MuGeN::PhysicalMapStripSet::AXISINDEX]};
    }
    if ($widget->layout eq 'interlace') {
	push @{$self->{strips}},new MuGeN::ResultStrip($widget,
						$self->{startbase},
						 $self->{endbase});
	push @{$self->{interlacecompanalresultwidgets}},{widget=>$widget,
							 strip=>$self->{strips}->[$#{$self->{strips}}]};
    }

}

sub area {
    my $self=shift;
    my $left_x=shift;
    if (defined $left_x) {
	$self->{x_left}=$left_x;
	$self->{y_top}=shift;
	$self->{width}=shift;
	$self->{height}=shift;
	
	my $stripset_height_in_lines=$self->height_in_lines;
	my $cur_height=$self->{y_top}+$self->{height}/$stripset_height_in_lines;
	_determine_view_mode(abs($self->{endbase}-
				 $self->{startbase}+1));
	
	foreach my $strip (@{$self->{strips}}) {
	    my $strip_height=$strip->height_in_lines/$stripset_height_in_lines*
		$self->{height};
	    $strip->offset($self->{x_left},$cur_height);
	    $strip->scale($self->{width},$strip_height);
	    $cur_height+=$strip_height;
	}
    } else {
	return ($self->{x_left},$self->{y_top},$self->{width},$self->{height});
    }
}

sub draw {
    my $self=shift;
    my $paintdevice=shift;
    $paintdevice->add_rectangle($self->{x_left},$self->{y_top},
				$self->{width},$self->{height},
				'filled'=>1,
				'color'=>$self->{backgroundcolor},
				'depth'=>999);
				
    _determine_view_mode(abs($self->{endbase}-
			     $self->{startbase}+1));

    foreach my $strip (@{$self->{strips}}) {
	$strip->draw($paintdevice);
    }
    foreach my $companalref (@{$self->{interlacecompanalresultwidgets}}) {
	$companalref->{widget}->draw($paintdevice,$companalref->{strip});
    }
    foreach my $companalref (@{$self->{scalablecompanalresultwidgets}}) {
	$companalref->{widget}->draw($paintdevice,$companalref->{strip});
    }
    foreach my $companalref (@{$self->{axiscompanalresultwidgets}}) {
	$companalref->{widget}->draw($paintdevice,$companalref->{strip});
    }
}

sub height_in_lines {
    my $self=shift;

    _determine_view_mode(abs($self->{endbase}-
			     $self->{startbase}+1));

    my $lines=2;
    foreach my $strip (@{$self->{strips}}) {
	$lines+=$strip->height_in_lines;
    }

    return $lines;
}

sub set_background_color{
    my $self=shift;
    my $color=shift;

    $self->{backgroundcolor}=$color;
}

1
